#include "Vector.h"
#include "Scene.h"
#include "trax.hpp"

// Returns a color using simple Lambertian shading
Color shade(const HitRecord &hr, const Ray &ray, const Vector& hitPoint, const Scene& scene)
{
  
  //float epsilon = loadf(0, 18);
  // Hard-code epsilon for assignment 2
  float epsilon = 1e-3f;
  
  // Did the ray hit anything?
  if(hr.didHit())
    {
      Vector normal = scene.spheres[hr.getID()].normal(hitPoint);
      float costheta = Dot(normal, ray.direction());

      // Do we need to flip the normal?
      if(costheta > 0.0f)
	normal = -normal;

      Color retVal = scene.ambient * 0.3f;
      for(int i=0; i < scene.numLights; i++)
	{
	  Color lightColor;
	  Vector lightDir;
	  float lightDistance = scene.lights[i].getLight(lightColor, lightDir, hitPoint);
	  float cosphi = Dot(normal, lightDir);

	  // Is the hit point on the right side of the object to see the light?
	  if(cosphi > 0.0f)
	    {
	      Vector o = hitPoint + (normal * epsilon); // offset to prevent self-shadow
	      Ray shadow(o, lightDir);
	      HitRecord shadowHit(lightDistance);

	      // Check if the shadow ray makes it to the light
	      for(int k=0; k < scene.numSpheres; k++)
		{
		  scene.spheres[k].intersect(shadowHit, shadow);
		}
	      if(!shadowHit.didHit())
		{
		  retVal += (lightColor * cosphi * 0.7f);
		}
	    }
	}
      retVal *= scene.materials[scene.spheres[hr.getID()].matl_id];
      
      return retVal;
    }
  else
    // Primary ray didn't hit anything
    return scene.background;
}


