#ifndef PinholeCamera_h
#define PinholeCamera_h

#include "Vector.h"
#include "Ray.h"

class PinholeCamera{
 public:
  PinholeCamera(const Vector& eye, const Vector& _gaze, const Vector& up, float ulen, float aspect_ratio);
  void makeRay(Ray& ray, float x, float y) const;

 private:
  Vector eye;
  Vector gaze;
  Vector up;
  float ulen;
  float aspect_ratio;

  Vector u;
  Vector v;
};



PinholeCamera::PinholeCamera(const Vector& _eye, const Vector& _gaze, const Vector& _up,
                             float _ulen, float _aspect_ratio)
: eye(_eye), gaze(_gaze), up(_up), ulen(_ulen), aspect_ratio(_aspect_ratio)
{
  gaze.normalize();
  u = Cross(gaze, up);
  v = Cross(u, gaze);
  u.normalize();
  u *= ulen;
  float vlen = ulen/aspect_ratio;
  v.normalize(); 
  v *= vlen;
}

void PinholeCamera::makeRay(Ray& ray, float x, float y) const
{
  Vector direction = gaze+u*x+v*y;
  direction.normalize();
  ray = Ray(eye, direction);
}

#endif
