#include "Color.h"
#include "Image.h"
#include "Ray.h"
#include "Sphere.h"
#include "PinholeCamera.h"
#include "PointLight.h"
#include "LambertianMaterial.h"
#include "Scene.h"
#include "trax.hpp"

void trax_main()
{
  Color materials[4];
  Sphere spheres[4];
  PointLight lights[2];

  int xres = loadi( 0, 1 );
  int yres = loadi( 0, 4 ); 
  int start_fb = loadi( 0, 7 );

  Image image(xres, yres, start_fb);
 
  
  PinholeCamera camera(Vector(-24.f, -2.f, 5.2f),
		       Vector(0.986794f, 0.118415f, -0.110521f),
		       Vector(0.f, 0.f, 1.f),
		       0.194f, (float)xres / (float)yres); // FOV of 22 degrees (for 1.0 aspect)

  materials[0] = Color(0.1f, 0.3f, 0.9f);
  materials[1] = Color(1.f, 0.2f, 0.2f);
  materials[2] = Color(1.f, 0.9f, 0.1f);
  materials[3] = Color(.4f, .4f, .7f);
  
  spheres[0] = Sphere(Vector(1.5f, 3.5f, 4.f), 2.4f, 0, 0);
  spheres[1] = Sphere(Vector(-0.5f, -1.5f, 2.f), 1.8f, 1, 1);
  spheres[2] = Sphere(Vector(0.5f, 1.0f, 1.f), 1.f, 2, 2);
  spheres[3] = Sphere(Vector(0.5f, 1.0f, -100.f), 100.f, 3, 3);

  lights[0] = PointLight(Vector(-30.f, -20.f, 80.f), Color(.7f,.9f,.9f));
  lights[1] = PointLight(Vector(-20.f, -50.f, 40.f), Color(.6f,.1f,.1f));
  
  Color background(0.5f, 0.8f, 0.9f);
  Color ambient(0.6f, 0.6f, 0.6f);

  Scene scene(spheres, 4, materials, 4, lights, 2, background, ambient);
  
  for(int pix = atomicinc(0); pix < xres*yres; pix = atomicinc(0))
    {
      int i = pix / xres;
      int j = pix % xres;

      Color result;
      
      float x = 2.0f * (j - xres/2.f + 0.5f)/xres;
      float y = 2.0f * (i - yres/2.f + 0.5f)/yres;
      
      // cast a single ray for this pixel
      Ray ray;
      camera.makeRay(ray, x, y);
      HitRecord hit(100000.f);
      // intersect that ray with each sphere
      for(int k=0; k < 4; k++)
	{
	  spheres[k].intersect(hit, ray);
	}

      // shade the pixel after finding closest intersection
      Vector hitPoint = ray.origin() + (hit.minT() * ray.direction());
      result = shade(hit, ray, hitPoint, scene);
      image.set(i, j, result);
  }
}



